package config

import (
	"context"
	"egg-im/pkg/k8sutil"
	"egg-im/pkg/logger"
	"go.uber.org/zap"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"os"
	"strconv"
)

const (
	//RPCListenAddr = ":8000"
	//TCPListenAddr = ":8080"
	//WSListenAddr  = ":8001"
	RPCListenAddr = ":8000"
	TCPListenAddr = ":8001"
	WSListenAddr  = ":8002"
)

var (
	NameSpace     string = "egg-im"
	MySQL         string = "root:Fnuo123com@@tcp(119.23.182.117:3306)/egg-im?charset=utf8&parseTime=true&loc=Local"
	RedisIP       string = "120.24.28.6:32572"
	RedisPassword string = ""

	LocalAddr            string
	PushRoomSubscribeNum int
	PushAllSubscribeNum  int
)

func Init() {
	k8sClient, err := k8sutil.GetK8sClient()
	if err != nil {
		panic(err)
	}
	configmap, err := k8sClient.CoreV1().ConfigMaps(NameSpace).Get(context.TODO(), "config", metav1.GetOptions{})
	if err != nil {
		panic(err)
	}

	MySQL = configmap.Data["mysql"]
	RedisIP = configmap.Data["redisIP"]
	RedisPassword = configmap.Data["redisPassword"]
	PushRoomSubscribeNum, _ = strconv.Atoi(configmap.Data["pushRoomSubscribeNum"])
	if PushRoomSubscribeNum == 0 {
		panic("PushRoomSubscribeNum == 0")
	}
	PushAllSubscribeNum, _ = strconv.Atoi(configmap.Data["pushAllSubscribeNum"])
	if PushRoomSubscribeNum == 0 {
		panic("PushAllSubscribeNum == 0")
	}

	LocalAddr = os.Getenv("POD_IP") + RPCListenAddr

	logger.Level = zap.DebugLevel
	logger.Target = logger.Console
}
